/*
    Copyright 2020. Huawei Technologies Co., Ltd. All rights reserved.
    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at
    http://www.apache.org/licenses/LICENSE-2.0
    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
*/

import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'dart:async';

import 'package:agconnect_auth/agconnect_auth.dart';

class PageSelfBuildAuth extends StatefulWidget {
  @override
  State<StatefulWidget> createState() {
    return _PageSelfBuildAuthState();
  }
}

class _PageSelfBuildAuthState extends State<PageSelfBuildAuth> {
  String _log = '';
  TextEditingController _selfBuildController = TextEditingController();

  @override
  void initState() {
    super.initState();
    _getCurrentUser();
  }

  _getCurrentUser() async {
    AGCAuth.instance.currentUser.then((value) {
      setState(() {
        _log = 'current user = ${value?.uid} , ${value?.providerId}';
      });
    });
  }

  _signIn() async {
    bool result = await _showSelfBuildDialog(VerifyCodeAction.registerLogin);
    if (result == null) {
      print("cancel");
      return;
    }
    String token = _selfBuildController.text;
    AGCAuthCredential credential = SelfBuildAuthProvider.credentialWithToken(token);
    AGCAuth.instance.signIn(credential).then((value) {
      setState(() {
        _log = 'signIn = ${value.user.uid} , ${value.user.providerId}';
      });
    });
  }

  _signOut() {
    AGCAuth.instance.signOut().then((value) {
      setState(() {
        _log = 'signOut';
      });
    }).catchError((error)=>print(error));
  }

  _deleteUser() {
    AGCAuth.instance.deleteUser().then((value) {
      setState(() {
        _log = 'deleteUser';
      });
    }).catchError((error)=>print(error));
  }

  _link() async {
    AGCUser user = await AGCAuth.instance.currentUser;
    if (user == null) {
      print("no user signed in");
      return;
    }
    bool result = await _showSelfBuildDialog(VerifyCodeAction.registerLogin);
    if (result == null) {
      print("cancel");
      return;
    }
    String token = _selfBuildController.text;
    AGCAuthCredential credential = SelfBuildAuthProvider.credentialWithToken(token);
    SignInResult signInResult = await user.link(credential).catchError((error){
      print(error);
    });
    setState(() {
      _log = 'link self build = ${signInResult?.user?.uid}';
    });
  }

  _unlink() async {
    AGCUser user = await AGCAuth.instance.currentUser;
    if (user == null) {
      print("no user signed in");
      return;
    }
    SignInResult result = await user.unlink(AuthProviderType.selfBuild).catchError((error){
      print(error);
    });
    setState(() {
      _log = 'unlink self build = ${result?.user?.uid}';
    });
  }

  Future<bool> _showSelfBuildDialog(VerifyCodeAction action) {
    return showDialog<bool>(
      context: context,
      builder: (ctx) {
        return AlertDialog(
          title: Text("Input"),
          content: Column(
            mainAxisSize: MainAxisSize.min,
            children: <Widget>[
              CupertinoTextField(controller: _selfBuildController,placeholder: 'jwt',),
            ],
          ),
          actions: <Widget>[
            FlatButton(
              child: Text("CANCEL"),
              onPressed: () => Navigator.of(context).pop(),
            ),
            FlatButton(
              child: Text("OK"),
              onPressed: () {
                Navigator.of(context).pop(true);
              },
            ),
          ],
        );
      },
    );
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        title: Text('Self Build Auth'),
      ),
      body: Center(
          child: ListView(children: <Widget>[
            Text(_log),
            CupertinoButton(child: Text('getCurrentUser'), onPressed: _getCurrentUser),
            CupertinoButton(child: Text('signIn'), onPressed: _signIn),
            CupertinoButton(child: Text('signOut'), onPressed: _signOut),
            CupertinoButton(child: Text('deleteUser'), onPressed: _deleteUser),
            CupertinoButton(child: Text('link self build'), onPressed: _link),
            CupertinoButton(child: Text('unlink self build'), onPressed: _unlink),
          ],)
      ),
    );
  }
}