/*
    Copyright 2020. Huawei Technologies Co., Ltd. All rights reserved.
    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at
    http://www.apache.org/licenses/LICENSE-2.0
    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
*/

#import "AGConnectAuthPlugin.h"
#import <AGConnectAuth/AGConnectAuth.h>

@interface AGConnectAuthPlugin ()

@property (nonatomic) FlutterEventChannel *eventChannel;
@property (nonatomic) FlutterEventSink eventSink;

@end

@implementation AGConnectAuthPlugin
+ (void)registerWithRegistrar:(NSObject<FlutterPluginRegistrar>*)registrar {
    FlutterMethodChannel* channel = [FlutterMethodChannel methodChannelWithName:@"com.huawei.flutter/agconnect_auth"
                                                                binaryMessenger:[registrar messenger]];
    AGConnectAuthPlugin* instance = [[AGConnectAuthPlugin alloc] init];
    [registrar addMethodCallDelegate:instance channel:channel];
    [registrar addApplicationDelegate:instance];
    
    FlutterEventChannel* eventChannel = [FlutterEventChannel eventChannelWithName:@"com.huawei.flutter.event/agconnect_auth" binaryMessenger:[registrar messenger]];
    [eventChannel setStreamHandler:instance];
    instance.eventChannel = eventChannel;
}

#pragma mark - AppDelegate
- (BOOL)application:(UIApplication *)application didFinishLaunchingWithOptions:(NSDictionary *)launchOptions {
    @try {
        [AGCInstance startUp];
        [self addTokenChangeListener];
    } @catch (NSException *exception) {
        NSLog(@"Exception: AGConnectRemoteConfigPlugin init failed, exception name: %@, exception reason: %@", exception.name, exception.reason);
        @throw exception;
    } @finally {
        
    }
    return YES;
}

- (void)addTokenChangeListener {
    [AGCAuth.getInstance addTokenListener:^(AGCTokenSnapshot * _Nonnull tokenSnapshot) {
        if (self.eventSink) {
            self.eventSink([self dictionaryFromTokenSnapshot:tokenSnapshot]);
        }
    }];
}

#pragma mark - FlutterStreamHandler
- (FlutterError *)onListenWithArguments:(id)arguments eventSink:(FlutterEventSink)events {
    self.eventSink = events;
    return nil;
}

- (FlutterError *)onCancelWithArguments:(id)arguments {
    self.eventSink = nil;
    return nil;
}

- (void)handleMethodCall:(FlutterMethodCall*)call result:(FlutterResult)result {
    if ([@"signIn" isEqualToString:call.method]) {
        AGCAuthCredential *credential = [self getCredentialFromArguments:call.arguments];
        [[[[AGCAuth getInstance] signIn:credential] addOnSuccessCallback:^(AGCSignInResult * _Nullable res) {
            result([self dictionaryFromUser:res.user]);
        }] addOnFailureCallback:^(NSError * _Nonnull error) {
            result([self getFlutterError:error]);
        }];
        
    }else if ([@"signInAnonymously" isEqualToString:call.method]) {
        [[[[AGCAuth getInstance] signInAnonymously] addOnSuccessCallback:^(AGCSignInResult * _Nullable res) {
            result([self dictionaryFromUser:res.user]);
        }] addOnFailureCallback:^(NSError * _Nonnull error) {
            result([self getFlutterError:error]);
        }];
        
    }else if ([@"signOut" isEqualToString:call.method]) {
        [[AGCAuth getInstance] signOut];
        result(nil);
    }else if ([@"deleteUser" isEqualToString:call.method]) {
        [[AGCAuth getInstance] deleteUser];
        result(nil);
    }else if ([@"getCurrentUser" isEqualToString:call.method]) {
        AGCUser *user = [[AGCAuth getInstance] currentUser];
        result([self dictionaryFromUser:user]);
    }else if ([@"createUserWithEmail" isEqualToString:call.method]) {
        NSString *email = call.arguments[@"email"];
        NSString *password = call.arguments[@"password"];
        NSString *verifyCode = call.arguments[@"verifyCode"];
        [[[[AGCAuth getInstance] createUserWithEmail:email password:password verifyCode:verifyCode] addOnSuccessCallback:^(AGCSignInResult * _Nullable res) {
            result([self dictionaryFromUser:res.user]);
        }] addOnFailureCallback:^(NSError * _Nonnull error) {
            result([self getFlutterError:error]);
        }];
    }else if ([@"createUserWithPhone" isEqualToString:call.method]) {
        NSString *countryCode = call.arguments[@"countryCode"];
        NSString *phoneNumber = call.arguments[@"phoneNumber"];
        NSString *password = call.arguments[@"password"];
        NSString *verifyCode = call.arguments[@"verifyCode"];
        [[[[AGCAuth getInstance] createUserWithCountryCode:countryCode phoneNumber:phoneNumber password:password verifyCode:verifyCode] addOnSuccessCallback:^(AGCSignInResult * _Nullable res) {
            result([self dictionaryFromUser:res.user]);
        }] addOnFailureCallback:^(NSError * _Nonnull error) {
            result([self getFlutterError:error]);
        }];
    }else if ([@"resetPasswordWithEmail" isEqualToString:call.method]) {
        NSString *email = call.arguments[@"email"];
        NSString *password = call.arguments[@"password"];
        NSString *verifyCode = call.arguments[@"verifyCode"];
        [[[[AGCAuth getInstance] resetPasswordWithEmail:email newPassword:password verifyCode:verifyCode] addOnSuccessCallback:^(id  _Nullable res) {
            result(nil);
        }] addOnFailureCallback:^(NSError * _Nonnull error) {
            result([self getFlutterError:error]);
        }];
    }else if ([@"resetPasswordWithPhone" isEqualToString:call.method]) {
        NSString *countryCode = call.arguments[@"countryCode"];
        NSString *phoneNumber = call.arguments[@"phoneNumber"];
        NSString *password = call.arguments[@"password"];
        NSString *verifyCode = call.arguments[@"verifyCode"];
        [[[[AGCAuth getInstance] resetPasswordWithCountryCode:countryCode phoneNumber:phoneNumber newPassword:password verifyCode:verifyCode] addOnSuccessCallback:^(id  _Nullable res) {
            result(nil);
        }] addOnFailureCallback:^(NSError * _Nonnull error) {
            result([self getFlutterError:error]);
        }];
    }else if ([@"link" isEqualToString:call.method]) {
        AGCAuthCredential *credential = [self getCredentialFromArguments:call.arguments];
        [[[AGCAuth.getInstance.currentUser link:credential] addOnSuccessCallback:^(AGCSignInResult * _Nullable res) {
            result([self dictionaryFromUser:res.user]);
        }] addOnFailureCallback:^(NSError * _Nonnull error) {
            result([self getFlutterError:error]);
        }];
    }else if ([@"unlink" isEqualToString:call.method]) {
        NSInteger provider = [(NSNumber *)call.arguments[@"provider"] integerValue];
        [[[AGCAuth.getInstance.currentUser unlink:provider] addOnSuccessCallback:^(AGCSignInResult * _Nullable res) {
            result([self dictionaryFromUser:res.user]);
        }] addOnFailureCallback:^(NSError * _Nonnull error) {
            result([self getFlutterError:error]);
        }];
    }else if ([@"updateProfile" isEqualToString:call.method]) {
        AGCProfileRequest *request = [AGCProfileRequest new];
        request.displayName = call.arguments[@"displayName"];
        request.photoUrl = call.arguments[@"photoUrl"];
        [[[AGCAuth.getInstance.currentUser updateProfile:request] addOnSuccessCallback:^(id  _Nullable res) {
            result([self dictionaryFromUser:AGCAuth.getInstance.currentUser]);
        }] addOnFailureCallback:^(NSError * _Nonnull error) {
            result([self getFlutterError:error]);
        }];
    }else if ([@"updateEmail" isEqualToString:call.method]) {
        NSString *email = call.arguments[@"email"];
        NSString *verifyCode = call.arguments[@"verifyCode"];
        NSLocale *locale = [self getLocaleFromArguments:call.arguments];
        [[[AGCAuth.getInstance.currentUser updateEmail:email verifyCode:verifyCode locale:locale] addOnSuccessCallback:^(id  _Nullable res) {
            result([self dictionaryFromUser:AGCAuth.getInstance.currentUser]);
        }] addOnFailureCallback:^(NSError * _Nonnull error) {
            result([self getFlutterError:error]);
        }];
    }else if ([@"updatePhone" isEqualToString:call.method]) {
        NSString *countryCode = call.arguments[@"countryCode"];
        NSString *phoneNumber = call.arguments[@"phoneNumber"];
        NSString *verifyCode = call.arguments[@"verifyCode"];
        NSLocale *locale = [self getLocaleFromArguments:call.arguments];
        [[[AGCAuth.getInstance.currentUser updatePhoneWithCountryCode:countryCode phoneNumber:phoneNumber verifyCode:verifyCode locale:locale] addOnSuccessCallback:^(id  _Nullable res) {
            result([self dictionaryFromUser:AGCAuth.getInstance.currentUser]);
        }] addOnFailureCallback:^(NSError * _Nonnull error) {
            result([self getFlutterError:error]);
        }];
    }else if ([@"updatePassword" isEqualToString:call.method]) {
        NSString *password = call.arguments[@"password"];
        NSString *verifyCode = call.arguments[@"verifyCode"];
        NSInteger provider = [(NSNumber *)call.arguments[@"provider"] integerValue];
        [[[AGCAuth.getInstance.currentUser updatePassword:password verifyCode:verifyCode provider:provider] addOnSuccessCallback:^(id  _Nullable res) {
            result([self dictionaryFromUser:AGCAuth.getInstance.currentUser]);
        }] addOnFailureCallback:^(NSError * _Nonnull error) {
            result([self getFlutterError:error]);
        }];
    }else if ([@"getUserExtra" isEqualToString:call.method]) {
        [[[AGCAuth.getInstance.currentUser getUserExtra] addOnSuccessCallback:^(AGCUserExtra * _Nullable res) {
            NSDictionary *userExtra = @{
                @"createTime":res.createTime ?: [NSNull null],
                @"lastSignInTime":res.lastSignInTime ?: [NSNull null]};
            NSDictionary *user = [self dictionaryFromUser:AGCAuth.getInstance.currentUser];
            result(@{@"userExtra":userExtra, @"user":user ?: [NSNull null]});
        }] addOnFailureCallback:^(NSError * _Nonnull error) {
            result([self getFlutterError:error]);
        }];
    }else if ([@"getToken" isEqualToString:call.method]) {
        BOOL forceFresh = [(NSNumber *)call.arguments[@"forceRefresh"] boolValue];
        [[[AGCAuth.getInstance.currentUser getToken:forceFresh] addOnSuccessCallback:^(AGCTokenResult * _Nullable res) {
            NSDictionary *tokenResult = @{
                @"token":res.token ?: [NSNull null],
                @"expirePeriod":@(res.expirePeriod)};
            result(tokenResult);
        }] addOnFailureCallback:^(NSError * _Nonnull error) {
            result([self getFlutterError:error]);
        }];
    }else if ([@"requestEmailVerifyCode" isEqualToString:call.method]) {
        NSString *email = call.arguments[@"email"];
        AGCVerifyCodeSettings *settings = [self getSettingsFromArguments:call.arguments];
        [[[AGCEmailAuthProvider requestVerifyCodeWithEmail:email settings:settings] addOnSuccessCallback:^(AGCVerifyCodeResult * _Nullable res) {
            result(@{@"shortestInterval":res.shortestInterval ?: [NSNull null], @"validityPeriod":res.validityPeriod ?: [NSNull null]});
        }] addOnFailureCallback:^(NSError * _Nonnull error) {
            result([self getFlutterError:error]);
        }];
    }else if ([@"requestPhoneVerifyCode" isEqualToString:call.method]) {
        NSString *countryCode = call.arguments[@"countryCode"];
        NSString *phoneNumber = call.arguments[@"phoneNumber"];
        AGCVerifyCodeSettings *settings = [self getSettingsFromArguments:call.arguments];
        [[[AGCPhoneAuthProvider requestVerifyCodeWithCountryCode:countryCode phoneNumber:phoneNumber settings:settings] addOnSuccessCallback:^(AGCVerifyCodeResult * _Nullable res) {
            result(@{@"shortestInterval":res.shortestInterval ?: [NSNull null], @"validityPeriod":res.validityPeriod ?: [NSNull null]});
        }] addOnFailureCallback:^(NSError * _Nonnull error) {
            result([self getFlutterError:error]);
        }];
    } else {
        result(FlutterMethodNotImplemented);
    }
}

- (NSDictionary *)dictionaryFromTokenSnapshot:(AGCTokenSnapshot *)token {
    if (token) {
        return @{
            @"state":@(token.state),
            @"token":token.token ?: [NSNull null],
        };
    }else{
        return nil;
    }
}

- (NSDictionary *)dictionaryFromUser:(AGCUser *)user {
    if (user) {
        return @{
            @"isAnonymous":@(user.isAnonymous),
            @"uid":user.uid ?: [NSNull null],
            @"email":user.email ?: [NSNull null],
            @"phone":user.phone ?: [NSNull null],
            @"displayName":user.displayName ?: [NSNull null],
            @"photoUrl":user.photoUrl ?: [NSNull null],
            @"providerId":@(user.providerId),
            @"providerInfo":user.providerInfo ?: [NSNull null],
            @"emailVerified":@(user.emailVerified),
            @"passwordSet":@(user.passwordSetted),
        };
    }else{
        return nil;
    }
}

- (AGCVerifyCodeSettings *)getSettingsFromArguments:(NSDictionary *)arguments {
    NSDictionary *dict = arguments[@"settings"];
    if (dict) {
        AGCVerifyCodeAction action = [dict[@"action"] integerValue] == 0 ? AGCVerifyCodeActionRegisterLogin : AGCVerifyCodeActionResetPassword;
        NSLocale *locale = [self getLocaleFromArguments:dict];
        NSInteger interval = [dict[@"sendInterval"] integerValue];
        return [[AGCVerifyCodeSettings alloc] initWithAction:action locale:locale sendInterval:interval];
    }
    return nil;
}

- (NSLocale *)getLocaleFromArguments:(NSDictionary *)arguments {
    if (arguments) {
        NSString *localeLanguage = arguments[@"localeLanguage"];
        NSString *localeCountry = arguments[@"localeCountry"];
        if ([localeLanguage isKindOfClass:NSString.class] && [localeCountry isKindOfClass:NSString.class]) {
            return [NSLocale localeWithLocaleIdentifier:[NSString stringWithFormat:@"%@_%@",localeLanguage,localeCountry]];
        }
    }
    return nil;
}

- (AGCAuthCredential *)getCredentialFromArguments:(NSDictionary *)arguments {
    NSDictionary *dict = [arguments objectForKey:@"credential"];
    AGCAuthProviderType type = [[dict objectForKey:@"provider"] intValue];
    switch (type) {
        case AGCAuthProviderTypeSelfBuild:
        {
            return [AGCSelfBuildAuthProvider credentialWithToken:dict[@"token"] autoCreateUser:[dict[@"autoCreateUser"] boolValue]];
        }
            
        case AGCAuthProviderTypePhone:
        {
            NSString *password = dict[@"password"] == [NSNull null] ? nil : dict[@"password"];
            NSString *verifyCode = dict[@"verifyCode"] == [NSNull null] ? nil : dict[@"verifyCode"];
            return [AGCPhoneAuthProvider credentialWithCountryCode:dict[@"countryCode"] phoneNumber:dict[@"phoneNumber"] password:password verifyCode:verifyCode];
        }
            
        case AGCAuthProviderTypeEmail:
        {
            NSString *password = dict[@"password"] == [NSNull null] ? nil : dict[@"password"];
            NSString *verifyCode = dict[@"verifyCode"] == [NSNull null] ? nil : dict[@"verifyCode"];
            return [AGCEmailAuthProvider credentialWithEmail:dict[@"email"] password:password verifyCode:verifyCode];
        }
            
        default:
            return nil;
    }
}

- (FlutterError *)getFlutterError:(NSError *)error {
    NSInteger exceptionCode = error.code;
    if ([error isKindOfClass:AGCAuthError.class]) {
        exceptionCode = [self getAuthExceptionCode:error.code];
    }
    return [FlutterError errorWithCode:[NSString stringWithFormat:@"%ld",(long)error.code]
                               message:error.localizedDescription
                               details:@{@"exceptionCode":@(exceptionCode)}];
}

- (NSInteger)getAuthExceptionCode:(NSInteger)code {
    switch (code) {
        case AGCAuthErrorCodeNullToken:
            return 0;
        case AGCAuthErrorCodeNotSignIn:
            return 1;
        case AGCAuthErrorCodeUserLinked:
            return 2;
        case AGCAuthErrorCodeUserUnlinked:
            return 3;
        case AGCAuthErrorCodeAlreadySignInUser:
            return 4;
        case AGCAuthErrorCodeEmailVerificationIsEmpty:
            return 5;
        case AGCAuthErrorCodePhoneVerificationIsEmpty:
            return 6;
        case AGCAuthErrorCodeInvalidEmail:
            return 7;
        case AGCAuthErrorCodeInvalidPhone:
            return 8;
        case AGCAuthErrorCodeGetUidError:
            return 9;
        case AGCAuthErrorCodeUidProductIDNotMatch:
            return 10;
        case AGCAuthErrorCodeGetUserInfoError:
            return 11;
        case AGCAuthErrorCodeAuthMethodNotSupport:
            return 12;
        case AGCAuthErrorCodeProductStatusError:
            return 13;
        case AGCAuthErrorCodePasswordVerificationCodeOverLimit:
            return 14;
        case AGCAuthErrorCodeInvalidToken:
            return 15;
        case AGCAuthErrorCodeInvalidAccessToken:
            return 16;
        case AGCAuthErrorCodeInvalidRefreshToken:
            return 17;
        case AGCAuthErrorCodeTokenAndProductIdNotMatch:
            return 18;
        case AGCAuthErrorCodeAuthMethodIsDisabled:
            return 19;
        case AGCAuthErrorCodeFailToGetThirdUserInfo:
            return 20;
        case AGCAuthErrorCodeFailToGetThirdUserUnionId:
            return 21;
        case AGCAuthErrorCodeAccessTokenOverLimit:
            return 22;
        case AGCAuthErrorCodeFailToUserLink:
            return 23;
        case AGCAuthErrorCodeFailToUserUnlink:
            return 24;
        case AGCAuthErrorCodeAnonymousSigninOverLimit:
            return 25;
        case AGCAuthErrorCodeInvalidAppID:
            return 26;
        case AGCAuthErrorCodeInvalidAppSecret:
            return 27;
        case AGCAuthErrorCodeGetQQUserInfoError:
            return 28;
        case AGCAuthErrorCodeQQInfoResponseIsNull:
            return 29;
        case AGCAuthErrorCodeGetQQUidError:
            return 30;
        case AGCAuthErrorCodePasswordVerifyCodeError:
            return 31;
        case AGCAuthErrorCodeGoogleResponseNotEqualAppID:
            return 32;
        case AGCAuthErrorCodeSignInUserStatusError:
            return 33;
        case AGCAuthErrorCodeSignInUserPasswordError:
            return 34;
        case AGCAuthErrorCodeProviderUserHaveBeenLinked:
            return 35;
        case AGCAuthErrorCodeProviderHaveLinkedOneUser:
            return 36;
        case AGCAuthErrorCodeFailGetProviderUser:
            return 37;
        case AGCAuthErrorCodeCannotUnlinkOneProviderUser:
            return 38;
        case AGCAuthErrorCodeVerifyCodeIntervalLimit:
            return 39;
        case AGCAuthErrorCodeVerifyCodeEmpty:
            return 40;
        case AGCAuthErrorCodeVerifyCodeLanguageEmpty:
            return 41;
        case AGCAuthErrorCodeVerifyCodeReceiverEmpty:
            return 42;
        case AGCAuthErrorCodeVerifyCodeActionError:
            return 43;
        case AGCAuthErrorCodeVerifyCodeTimeLimit:
            return 44;
        case AGCAuthErrorCodeAccountPasswordSame:
            return 45;
        case AGCAuthErrorCodePasswordStrengthLow:
            return 46;
        case AGCAuthErrorCodeUpdatePasswordError:
            return 47;
        case AGCAuthErrorCodePasswordSameAsBefore:
            return 48;
        case AGCAuthErrorCodePasswordIsEmpty:
            return 49;
        case AGCAuthErrorCodePasswordTooLong:
            return 50;
        case AGCAuthErrorCodeSensitiveOperationTimeout:
            return 51;
        case AGCAuthErrorCodeAccountHaveBeenRegistered:
            return 52;
        case AGCAuthErrorCodeUpdateAccountError:
            return 53;
        case AGCAuthErrorCodeUserNotRegistered:
            return 54;
        case AGCAuthErrorCodeVerifyCodeError:
            return 55;
        case AGCAuthErrorCodeUserHaveBeenRegistered:
            return 56;
        case AGCAuthErrorCodeRegisterAccountIsEmpty:
            return 57;
        case AGCAuthErrorCodeVerifyCodeFormatError:
            return 58;
        case AGCAuthErrorCodeVerifyCodeAndPasswordBothNull:
            return 59;
        case AGCAuthErrorCodeSendEmailFail:
            return 60;
        case AGCAuthErrorCodeSendMessageFail:
            return 61;
        case AGCAuthErrorCodeConfigLockTimeError:
            return 62;
            
        default:
            return code;
    }
}

@end
