import 'package:flutter/services.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:agconnect_auth/agconnect_auth.dart';

void main() {
  TestWidgetsFlutterBinding.ensureInitialized();

  group('authTest', () {
    const MethodChannel channel = MethodChannel('com.huawei.flutter/agconnect_auth');
    MethodCall mockCall;
    setUp(() {
      channel.setMockMethodCallHandler((MethodCall methodCall) async {
        mockCall = methodCall;
        switch (methodCall.method) {
          case 'signInAnonymously':
          case 'signIn':
          case 'getCurrentUser':
          case 'createUserWithEmail':
          case 'createUserWithPhone':
            return {
        'isAnonymous':false,
        'uid':'test_uid',
        'email':'test_email',
        'phone':'test_phone',
        'displayName':'test_name',
        'photoUrl':'test_photo',
        'providerId':11,
        'providerInfo':[{'k1':'v1'}],
        'emailVerified':1,
        'passwordSet':1,
        };
          case 'signOut':
            return null;
          case 'deleteUser':
            return null;
          case 'resetPasswordWithEmail':
            return null;
          case 'resetPasswordWithPhone':
            return null;
        }
        throw PlatformException(code: '0', message: 'Unknown method call, please update test.');
      });
      mockCall = null;
    });

    tearDown(() {
      channel.setMockMethodCallHandler(null);
    });

    test('signInAnonymously', () async {
      SignInResult res = await AGCAuth.instance.signInAnonymously();
      expect(mockCall.arguments, null);
      expect(res.user.uid, 'test_uid');
    });

    test('signInWithEmail', () async {
      AGCAuthCredential credential = EmailAuthProvider.credentialWithPassword('test_email', 'test_password');
      SignInResult res = await AGCAuth.instance.signIn(credential);
      expect(mockCall.arguments['credential'], {'provider':12, 'email':'test_email', 'password':'test_password', 'verifyCode':null});
      expect(res.user.uid, 'test_uid');
    });

    test('signInWithPhone', () async {
      AGCAuthCredential credential = PhoneAuthProvider.credentialWithVerifyCode('test_countryCode', 'test_phone', 'test_verifyCode', password: 'test_password');
      SignInResult res = await AGCAuth.instance.signIn(credential);
      expect(mockCall.arguments['credential'], {'provider':11, 'countryCode':'test_countryCode', 'phoneNumber':'test_phone', 'password':'test_password', 'verifyCode':'test_verifyCode'});
      expect(res.user.uid, 'test_uid');
    });

    test('signInWithPhone', () async {
      AGCAuthCredential credential = PhoneAuthProvider.credentialWithVerifyCode('test_countryCode', 'test_phone', 'test_verifyCode', password: 'test_password');
      SignInResult res = await AGCAuth.instance.signIn(credential);
      expect(mockCall.arguments['credential'], {'provider':11, 'countryCode':'test_countryCode', 'phoneNumber':'test_phone', 'password':'test_password', 'verifyCode':'test_verifyCode'});
      expect(res.user.uid, 'test_uid');
    });

    test('signOut', () async {
      await AGCAuth.instance.signOut();
      expect(mockCall.arguments, null);
      expect(mockCall.method, 'signOut');
    });

    test('deleteUser', () async {
      await AGCAuth.instance.deleteUser();
      expect(mockCall.arguments, null);
      expect(mockCall.method, 'deleteUser');
    });

    test('getCurrentUser', () async {
      AGCUser user =  await AGCAuth.instance.currentUser;
      expect(mockCall.arguments, null);
      expect(user.uid, 'test_uid');
    });

    test('createEmailUser', () async {
      EmailUser user = EmailUser('test_email', 'test_verifyCode', password: 'test_password');
      SignInResult res =  await AGCAuth.instance.createEmailUser(user);
      expect(mockCall.arguments, {'email':'test_email', 'password':'test_password', 'verifyCode':'test_verifyCode'});
      expect(res.user.uid, 'test_uid');
    });

    test('createPhoneUser', () async {
      PhoneUser user = PhoneUser('test_countryCode', 'test_phone', 'test_verifyCode', password: 'test_password');
      SignInResult res =  await AGCAuth.instance.createPhoneUser(user);
      expect(mockCall.arguments, {'countryCode':'test_countryCode', 'phoneNumber':'test_phone', 'password':'test_password', 'verifyCode':'test_verifyCode'});
      expect(res.user.uid, 'test_uid');
    });

    test('resetPasswordWithEmail', () async {
      await AGCAuth.instance.resetPasswordWithEmail('test_email', 'test_password', 'test_verifyCode');
      expect(mockCall.arguments, {'email':'test_email', 'password':'test_password', 'verifyCode':'test_verifyCode'});
      expect(mockCall.method, 'resetPasswordWithEmail');
    });

    test('resetPasswordWithPhone', () async {
      await AGCAuth.instance.resetPasswordWithPhone('test_countryCode', 'test_phone', 'test_password', 'test_verifyCode');
      expect(mockCall.arguments, {'countryCode':'test_countryCode', 'phoneNumber':'test_phone', 'password':'test_password', 'verifyCode':'test_verifyCode'});
      expect(mockCall.method, 'resetPasswordWithPhone');
    });

  });

}
